@@ -5,6 +5,9 @@
     in a heap identifier, for use with user defined memory overrides
     (see XMALLOC, XFREE, XREALLOC).
 
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING
+    is enabled.
+
     \return 0 Returned upon successfully initializing the RSA structure for
     use with encryption and decryption
     \return BAD_FUNC_ARGS Returned if the RSA key pointer evaluates to NULL
@@ -24,10 +27,10 @@
     }
     \endcode
 
-    \sa wc_RsaInitCavium
     \sa wc_FreeRsaKey
+    \sa wc_RsaSetRNG
 */
-WOLFSSL_API int  wc_InitRsaKey(RsaKey* key, void* heap);
+int  wc_InitRsaKey(RsaKey* key, void* heap);
 
 /*!
     \ingroup RSA
@@ -37,6 +40,9 @@ WOLFSSL_API int  wc_InitRsaKey(RsaKey* key, void* heap);
     the device.  It also takes in a heap identifier, for use with user defined
     memory overrides (see XMALLOC, XFREE, XREALLOC).
 
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING
+    is enabled.
+
     \return 0 Returned upon successfully initializing the RSA structure for
     use with encryption and decryption
     \return BAD_FUNC_ARGS Returned if the RSA key pointer evaluates to NULL
@@ -49,7 +55,7 @@ WOLFSSL_API int  wc_InitRsaKey(RsaKey* key, void* heap);
     \param heap pointer to a heap identifier, for use with memory overrides,
     allowing custom handling of memory allocation. This heap will be the
     default used when allocating memory for use with this RSA object
-    \param devId ID to use with hardware device
+    \param devId ID to use with crypto callbacks or async hardware. Set to INVALID_DEVID (-2) if not used
 
     _Example_
     \code
@@ -70,12 +76,39 @@ WOLFSSL_API int  wc_InitRsaKey(RsaKey* key, void* heap);
     \endcode
 
     \sa wc_InitRsaKey
-    \sa wc_RsaInitCavium
     \sa wc_FreeRsaKey
+    \sa wc_RsaSetRNG
 */
-WOLFSSL_API int  wc_InitRsaKey_Id(RsaKey* key, unsigned char* id, int len,
+int  wc_InitRsaKey_Id(RsaKey* key, unsigned char* id, int len,
         void* heap, int devId);
 
+/*!
+    \ingroup RSA
+
+    \brief This function associates RNG with Key. It is needed when WC_RSA_BLINDING
+    is enabled.
+
+    \return 0 Returned upon success
+    \return BAD_FUNC_ARGS Returned if the RSA key, rng pointer evaluates to NULL
+
+    \param key pointer to the RsaKey structure to be associated
+    \param rng pointer to the WC_RNG structure to associate with
+
+    _Example_
+    \code
+    ret = wc_InitRsaKey(&key, NULL);
+    if (ret == 0) {
+        ret = wc_InitRng(&rng);
+    } else return -1;
+    if (ret == 0) {
+        ret = wc_RsaSetRNG(&key, &rng);
+    \endcode
+
+    \sa wc_InitRsaKey
+    \sa wc_RsaSetRNG
+*/
+int wc_RsaSetRNG(RsaKey* key, WC_RNG* rng);
+
 /*!
     \ingroup RSA
 
@@ -88,7 +121,7 @@ WOLFSSL_API int  wc_InitRsaKey_Id(RsaKey* key, unsigned char* id, int len,
     _Example_
     \code
     RsaKey enc;
-    wc_RsaInitKey(&enc, NULL); // not using heap hint. No custom memory
+    wc_InitRsaKey(&enc, NULL); // not using heap hint. No custom memory
     ... set key, do encryption
 
     wc_FreeRsaKey(&enc);
@@ -96,7 +129,52 @@ WOLFSSL_API int  wc_InitRsaKey_Id(RsaKey* key, unsigned char* id, int len,
 
     \sa wc_InitRsaKey
 */
-WOLFSSL_API int  wc_FreeRsaKey(RsaKey* key);
+int  wc_FreeRsaKey(RsaKey* key);
+
+/*!
+    \ingroup RSA
+
+    \brief Function that does the RSA operation directly with no padding. The input
+        size must match key size. Typically this is
+        used when padding is already done on the RSA input.
+
+    \return size On successfully encryption the size of the encrypted buffer
+    is returned
+    \return RSA_BUFFER_E RSA buffer error, output too small or input too large
+
+    \param in buffer to do operation on
+    \param inLen length of input buffer
+    \param out buffer to hold results
+    \param outSz gets set to size of result buffer. Should be passed in as length
+        of out buffer. If the pointer "out" is null then outSz gets set to the
+        expected buffer size needed and LENGTH_ONLY_E gets returned.
+    \param key initialized RSA key to use for encrypt/decrypt
+    \param type if using private or public key (RSA_PUBLIC_ENCRYPT,
+        RSA_PUBLIC_DECRYPT, RSA_PRIVATE_ENCRYPT, RSA_PRIVATE_DECRYPT)
+    \param rng initialized WC_RNG struct
+
+    _Example_
+    \code
+    int ret;
+    WC_RNG rng;
+    RsaKey key;
+    byte  in[256];
+    byte out[256];
+    word32 outSz = (word32)sizeof(out);
+    …
+
+    ret = wc_RsaDirect(in, (word32)sizeof(in), out, &outSz, &key,
+        RSA_PRIVATE_ENCRYPT, &rng);
+    if (ret < 0) {
+	    //handle error
+    }
+    \endcode
+
+    \sa wc_RsaPublicEncrypt
+    \sa wc_RsaPrivateDecrypt
+*/
+int wc_RsaDirect(byte* in, word32 inLen, byte* out, word32* outSz,
+        RsaKey* key, int type, WC_RNG* rng);
 
 /*!
     \ingroup RSA
@@ -109,8 +187,6 @@ WOLFSSL_API int  wc_FreeRsaKey(RsaKey* key);
     \return Success Upon successfully encrypting the input message, returns
     0 for success and less than zero for failure. Also returns the number
     bytes written to out by storing the value in outLen
-    \return -1 Returned if there is an error during RSA encryption and
-    hardware acceleration via Cavium is enabled
     \return BAD_FUNC_ARG Returned if any of the input parameters are invalid
     \return RSA_BUFFER_E Returned if the output buffer is too small to store
     the ciphertext
@@ -170,7 +246,7 @@ WOLFSSL_API int  wc_FreeRsaKey(RsaKey* key);
 
     \sa wc_RsaPrivateDecrypt
 */
-WOLFSSL_API int  wc_RsaPublicEncrypt(const byte* in, word32 inLen, byte* out,
+int  wc_RsaPublicEncrypt(const byte* in, word32 inLen, byte* out,
                                  word32 outLen, RsaKey* key, WC_RNG* rng);
 
 /*!
@@ -194,7 +270,7 @@ WOLFSSL_API int  wc_RsaPublicEncrypt(const byte* in, word32 inLen, byte* out,
 
     \sa wc_RsaPrivateDecrypt
 */
-WOLFSSL_API int  wc_RsaPrivateDecryptInline(byte* in, word32 inLen, byte** out,
+int  wc_RsaPrivateDecryptInline(byte* in, word32 inLen, byte** out,
                                         RsaKey* key);
 
 /*!
@@ -228,7 +304,7 @@ WOLFSSL_API int  wc_RsaPrivateDecryptInline(byte* in, word32 inLen, byte** out,
     \sa wc_RsaFunction
     \sa wc_RsaPrivateDecryptInline
 */
-WOLFSSL_API int  wc_RsaPrivateDecrypt(const byte* in, word32 inLen, byte* out,
+int  wc_RsaPrivateDecrypt(const byte* in, word32 inLen, byte* out,
                                   word32 outLen, RsaKey* key);
 
 /*!
@@ -261,7 +337,7 @@ WOLFSSL_API int  wc_RsaPrivateDecrypt(const byte* in, word32 inLen, byte* out,
 
     \sa wc_RsaPad
 */
-WOLFSSL_API int  wc_RsaSSL_Sign(const byte* in, word32 inLen, byte* out,
+int  wc_RsaSSL_Sign(const byte* in, word32 inLen, byte* out,
                             word32 outLen, RsaKey* key, WC_RNG* rng);
 
 /*!
@@ -281,7 +357,7 @@ WOLFSSL_API int  wc_RsaSSL_Sign(const byte* in, word32 inLen, byte* out,
     _Example_
     \code
     RsaKey key;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     int ret = 0;
     long e = 65537; // standard value to use for exponent
     wc_InitRsaKey(&key, NULL); // not using heap hint. No custom memory
@@ -299,7 +375,7 @@ WOLFSSL_API int  wc_RsaSSL_Sign(const byte* in, word32 inLen, byte* out,
     \sa wc_RsaSSL_Verify
     \sa wc_RsaSSL_Sign
 */
-WOLFSSL_API int  wc_RsaSSL_VerifyInline(byte* in, word32 inLen, byte** out,
+int  wc_RsaSSL_VerifyInline(byte* in, word32 inLen, byte** out,
                                     RsaKey* key);
 
 /*!
@@ -331,7 +407,7 @@ WOLFSSL_API int  wc_RsaSSL_VerifyInline(byte* in, word32 inLen, byte** out,
 
     \sa wc_RsaSSL_Sign
 */
-WOLFSSL_API int  wc_RsaSSL_Verify(const byte* in, word32 inLen, byte* out,
+int  wc_RsaSSL_Verify(const byte* in, word32 inLen, byte* out,
                               word32 outLen, RsaKey* key);
 
 /*!
@@ -379,8 +455,9 @@ WOLFSSL_API int  wc_RsaSSL_Verify(const byte* in, word32 inLen, byte* out,
     \endcode
 
     \sa wc_RsaPSS_Verify
+    \sa wc_RsaSetRNG
 */
-WOLFSSL_API int  wc_RsaPSS_Sign(const byte* in, word32 inLen, byte* out,
+int  wc_RsaPSS_Sign(const byte* in, word32 inLen, byte* out,
                                 word32 outLen, enum wc_HashType hash, int mgf,
                                 RsaKey* key, WC_RNG* rng);
 
@@ -388,9 +465,11 @@ WOLFSSL_API int  wc_RsaPSS_Sign(const byte* in, word32 inLen, byte* out,
     \ingroup RSA
 
     \brief Decrypt input signature to verify that the message was signed by key.
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING is enabled.
 
     \return Success Length of text on no error.
     \return MEMORY_E memory exception.
+    \return MP_EXPTMOD_E - When using fastmath and FP_MAX_BITS not set to at least 2 times the keySize (Example when using 4096-bit key set FP_MAX_BITS to 8192 or greater value)
 
     \param in The byte array to be decrypted.
     \param inLen The length of in.
@@ -430,16 +509,20 @@ WOLFSSL_API int  wc_RsaPSS_Sign(const byte* in, word32 inLen, byte* out,
     \sa wc_RsaPSS_Sign
     \sa wc_RsaPSS_VerifyInline
     \sa wc_RsaPSS_CheckPadding
+    \sa wc_RsaSetRNG
 */
-WOLFSSL_API int  wc_RsaPSS_Verify(byte* in, word32 inLen, byte* out,
+int  wc_RsaPSS_Verify(byte* in, word32 inLen, byte* out,
                                   word32 outLen, enum wc_HashType hash, int mgf,
                                   RsaKey* key);
 
 /*!
     \ingroup RSA
 
-    \brief Decrypt input signature to verify that the message was signed by RSA key.  
+    \brief Decrypt input signature to verify that the message was signed by RSA
+    key.
     The output uses the same byte array as the input.
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING
+    is enabled.
 
     \return >0 Length of text.
     \return <0 An error occurred.
@@ -485,10 +568,11 @@ WOLFSSL_API int  wc_RsaPSS_Verify(byte* in, word32 inLen, byte* out,
     \sa wc_RsaPSS_VerifyCheckInline_ex
     \sa wc_RsaPSS_CheckPadding
     \sa wc_RsaPSS_CheckPadding_ex
+    \sa wc_RsaSetRNG
 */
 
 
-WOLFSSL_API int  wc_RsaPSS_VerifyInline(byte* in, word32 inLen, byte** out,
+int  wc_RsaPSS_VerifyInline(byte* in, word32 inLen, byte** out,
                                         enum wc_HashType hash, int mgf,
                                         RsaKey* key);
 /*!
@@ -496,8 +580,9 @@ WOLFSSL_API int  wc_RsaPSS_VerifyInline(byte* in, word32 inLen, byte** out,
 
     \brief Verify the message signed with RSA-PSS.
     Salt length is equal to hash length.
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING is enabled.
 
-    \return  the length of the PSS data on success and negative indicates failure.
+    \return the length of the PSS data on success and negative indicates failure.
     \return MEMORY_E memory exception.
 
     \param in The byte array to be decrypted.
@@ -527,7 +612,7 @@ WOLFSSL_API int  wc_RsaPSS_VerifyInline(byte* in, word32 inLen, byte** out,
         digestSz = wc_HashGetDigestSize(WC_HASH_TYPE_SHA256);
         ret = wc_Hash(WC_HASH_TYPE_SHA256, message, sz, digest, digestSz);
     } else return -1;
-    
+
     if (ret == 0) {
         ret = wc_RsaPSS_Sign(digest, digestSz, pSignature, pSignatureSz,
                 WC_HASH_TYPE_SHA256, WC_MGF1SHA256, &key, &rng);
@@ -552,9 +637,10 @@ WOLFSSL_API int  wc_RsaPSS_VerifyInline(byte* in, word32 inLen, byte** out,
     \sa wc_RsaPSS_VerifyCheckInline_ex
     \sa wc_RsaPSS_CheckPadding
     \sa wc_RsaPSS_CheckPadding_ex
+    \sa wc_RsaSetRNG
 */
 
-WOLFSSL_API int  wc_RsaPSS_VerifyCheck(byte* in, word32 inLen,
+int  wc_RsaPSS_VerifyCheck(byte* in, word32 inLen,
                                byte* out, word32 outLen,
                                const byte* digest, word32 digestLen,
                                enum wc_HashType hash, int mgf,
@@ -563,8 +649,9 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheck(byte* in, word32 inLen,
     \ingroup RSA
 
     \brief Verify the message signed with RSA-PSS.
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING is enabled.
 
-    \return  the length of the PSS data on success and negative indicates failure.
+    \return the length of the PSS data on success and negative indicates failure.
     \return MEMORY_E memory exception.
 
     \param in The byte array to be decrypted.
@@ -576,8 +663,8 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheck(byte* in, word32 inLen,
     \param hash Hash algorithm.
     \param mgf Mask generation function.
     \param saltLen  Length of salt used. RSA_PSS_SALT_LEN_DEFAULT (-1) indicates salt
-           length is the same as the hash length. RSA_PSS_SALT_LEN_DISCOVER
-           indicates salt length is determined from the data.
+    length is the same as the hash length. RSA_PSS_SALT_LEN_DISCOVER
+    indicates salt length is determined from the data.
 
     \param key Public RSA key.
 
@@ -598,7 +685,7 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheck(byte* in, word32 inLen,
         digestSz = wc_HashGetDigestSize(WC_HASH_TYPE_SHA256);
         ret = wc_Hash(WC_HASH_TYPE_SHA256, message, sz, digest, digestSz);
     } else return -1;
-    
+
     if (ret == 0) {
         ret = wc_RsaPSS_Sign(digest, digestSz, pSignature, pSignatureSz,
                 WC_HASH_TYPE_SHA256, WC_MGF1SHA256, &key, &rng);
@@ -623,8 +710,9 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheck(byte* in, word32 inLen,
     \sa wc_RsaPSS_VerifyCheckInline_ex
     \sa wc_RsaPSS_CheckPadding
     \sa wc_RsaPSS_CheckPadding_ex
+    \sa wc_RsaSetRNG
 */
-WOLFSSL_API int  wc_RsaPSS_VerifyCheck_ex(byte* in, word32 inLen,
+int  wc_RsaPSS_VerifyCheck_ex(byte* in, word32 inLen,
                                byte* out, word32 outLen,
                                const byte* digest, word32 digestLen,
                                enum wc_HashType hash, int mgf, int saltLen,
@@ -634,9 +722,11 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheck_ex(byte* in, word32 inLen,
     \ingroup RSA
 
     \brief Verify the message signed with RSA-PSS.
-    The input buffer is reused for the output buffer. 
+    The input buffer is reused for the output buffer.
     Salt length is equal to hash length.
 
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING is enabled.
+
     \return the length of the PSS data on success and negative indicates failure.
 
     \param in The byte array to be decrypted.
@@ -665,7 +755,7 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheck_ex(byte* in, word32 inLen,
         digestSz = wc_HashGetDigestSize(WC_HASH_TYPE_SHA256);
         ret = wc_Hash(WC_HASH_TYPE_SHA256, message, sz, digest, digestSz);
     } else return -1;
-    
+
     if (ret == 0) {
         ret = wc_RsaPSS_Sign(digest, digestSz, pSignature, pSignatureSz,
                 WC_HASH_TYPE_SHA256, WC_MGF1SHA256, &key, &rng);
@@ -690,8 +780,9 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheck_ex(byte* in, word32 inLen,
     \sa wc_RsaPSS_VerifyCheckInline_ex
     \sa wc_RsaPSS_CheckPadding
     \sa wc_RsaPSS_CheckPadding_ex
+    \sa wc_RsaSetRNG
 */
-WOLFSSL_API int  wc_RsaPSS_VerifyCheckInline(byte* in, word32 inLen, byte** out,
+int  wc_RsaPSS_VerifyCheckInline(byte* in, word32 inLen, byte** out,
                                const byte* digest, word32 digentLen,
                                enum wc_HashType hash, int mgf,
                                RsaKey* key);
@@ -699,7 +790,8 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheckInline(byte* in, word32 inLen, byte** out,
     \ingroup RSA
 
     \brief Verify the message signed with RSA-PSS.
-    The input buffer is reused for the output buffer. 
+    The input buffer is reused for the output buffer.
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING is enabled.
 
     \return the length of the PSS data on success and negative indicates failure.
 
@@ -711,8 +803,8 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheckInline(byte* in, word32 inLen, byte** out,
     \param hash The hash type to be in message
     \param mgf Mask Generation Function Identifiers
     \param saltLen  Length of salt used. RSA_PSS_SALT_LEN_DEFAULT (-1) indicates salt
-           length is the same as the hash length. RSA_PSS_SALT_LEN_DISCOVER
-           indicates salt length is determined from the data.
+    length is the same as the hash length. RSA_PSS_SALT_LEN_DISCOVER
+    indicates salt length is determined from the data.
     \param key The key to use for verification.
 
     _Example_
@@ -732,7 +824,7 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheckInline(byte* in, word32 inLen, byte** out,
         digestSz = wc_HashGetDigestSize(WC_HASH_TYPE_SHA256);
         ret = wc_Hash(WC_HASH_TYPE_SHA256, message, sz, digest, digestSz);
     } else return -1;
-    
+
     if (ret == 0) {
         ret = wc_RsaPSS_Sign(digest, digestSz, pSignature, pSignatureSz,
                 WC_HASH_TYPE_SHA256, WC_MGF1SHA256, &key, &rng);
@@ -757,8 +849,9 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheckInline(byte* in, word32 inLen, byte** out,
     \sa wc_RsaPSS_VerifyCheckInline
     \sa wc_RsaPSS_CheckPadding
     \sa wc_RsaPSS_CheckPadding_ex
+    \sa wc_RsaSetRNG
 */
-WOLFSSL_API int  wc_RsaPSS_VerifyCheckInline_ex(byte* in, word32 inLen, byte** out,
+int  wc_RsaPSS_VerifyCheckInline_ex(byte* in, word32 inLen, byte** out,
                                const byte* digest, word32 digentLen,
                                enum wc_HashType hash, int mgf, int saltLen,
                                RsaKey* key);
@@ -768,17 +861,18 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheckInline_ex(byte* in, word32 inLen, byte** o
 
     \brief Checks the PSS data to ensure that the signature matches.
     Salt length is equal to hash length.
+    The key has to be associated with RNG by wc_RsaSetRNG when WC_RSA_BLINDING is enabled.
 
     \return BAD_PADDING_E when the PSS data is invalid, BAD_FUNC_ARG when
     NULL is passed in to in or sig or inSz is not the same as the hash
     algorithm length and 0 on success.
     \return MEMORY_E memory exception.
 
-    \param in        Hash of the data that is being verified.
-    \param inSz      Length of hash.
-    \param sig       Buffer holding PSS data.
-    \param sigSz     Size of PSS data.
-    \param hashType  Hash algorithm.
+    \param in Hash of the data that is being verified.
+    \param inSz Length of hash.
+    \param sig Buffer holding PSS data.
+    \param sigSz Size of PSS data.
+    \param hashType Hash algorithm.
 
     _Example_
     \code
@@ -820,8 +914,9 @@ WOLFSSL_API int  wc_RsaPSS_VerifyCheckInline_ex(byte* in, word32 inLen, byte** o
     \sa wc_RsaPSS_VerifyCheckInline
     \sa wc_RsaPSS_VerifyCheckInline_ex
     \sa wc_RsaPSS_CheckPadding_ex
+    \sa wc_RsaSetRNG
 */
-WOLFSSL_API int  wc_RsaPSS_CheckPadding(const byte* in, word32 inLen, byte* sig,
+int  wc_RsaPSS_CheckPadding(const byte* in, word32 inLen, byte* sig,
                                         word32 sigSz,
                                         enum wc_HashType hashType);
 /*!
@@ -843,7 +938,7 @@ WOLFSSL_API int  wc_RsaPSS_CheckPadding(const byte* in, word32 inLen, byte* sig,
     \param saltLen   Length of salt used. RSA_PSS_SALT_LEN_DEFAULT (-1) indicates salt
         length is the same as the hash length. RSA_PSS_SALT_LEN_DISCOVER
         indicates salt length is determined from the data.
-    \param bits      igonred
+    \param bits      Can be used to calculate salt size in FIPS case
 
     _Example_
     \code
@@ -886,9 +981,8 @@ WOLFSSL_API int  wc_RsaPSS_CheckPadding(const byte* in, word32 inLen, byte* sig,
     \sa wc_RsaPSS_VerifyCheckInline_ex
     \sa wc_RsaPSS_CheckPadding
 */
-WOLFSSL_API int  wc_RsaPSS_CheckPadding_ex(const byte* in, word32 inLen, byte* sig,
-                                        word32 sigSz,
-                                        enum wc_HashType hashType, int saltLen, int bits);
+int  wc_RsaPSS_CheckPadding_ex(const byte* in, word32 inLen, byte* sig,
+                word32 sigSz, enum wc_HashType hashType, int saltLen, int bits);
 /*!
     \ingroup RSA
 
@@ -906,9 +1000,8 @@ WOLFSSL_API int  wc_RsaPSS_CheckPadding_ex(const byte* in, word32 inLen, byte* s
     \sa wc_InitRsaKey
     \sa wc_InitRsaKey_ex
     \sa wc_MakeRsaKey
-    \sa XMEMSET
 */
-WOLFSSL_API int  wc_RsaEncryptSize(RsaKey* key);
+int  wc_RsaEncryptSize(RsaKey* key);
 
 /*!
     \ingroup RSA
@@ -951,8 +1044,8 @@ WOLFSSL_API int  wc_RsaEncryptSize(RsaKey* key);
     \sa wc_RsaPublicKeyDecode
     \sa wc_MakeRsaKey
 */
-WOLFSSL_API int  wc_RsaPrivateKeyDecode(const byte* input, word32* inOutIdx,
-                                                               RsaKey*, word32);
+int  wc_RsaPrivateKeyDecode(const byte* input, word32* inOutIdx,
+                            RsaKey* key, word32 inSz);
 
 /*!
     \ingroup RSA
@@ -1000,8 +1093,8 @@ WOLFSSL_API int  wc_RsaPrivateKeyDecode(const byte* input, word32* inOutIdx,
 
     \sa wc_RsaPublicKeyDecodeRaw
 */
-WOLFSSL_API int  wc_RsaPublicKeyDecode(const byte* input, word32* inOutIdx,
-                                                               RsaKey*, word32);
+int  wc_RsaPublicKeyDecode(const byte* input, word32* inOutIdx,
+                           RsaKey* key, word32 inSz);
 
 /*!
     \ingroup RSA
@@ -1045,7 +1138,7 @@ WOLFSSL_API int  wc_RsaPublicKeyDecode(const byte* input, word32* inOutIdx,
 
     \sa wc_RsaPublicKeyDecode
 */
-WOLFSSL_API int  wc_RsaPublicKeyDecodeRaw(const byte* n, word32 nSz,
+int  wc_RsaPublicKeyDecodeRaw(const byte* n, word32 nSz,
                                         const byte* e, word32 eSz, RsaKey* key);
 
 /*!
@@ -1054,7 +1147,7 @@ WOLFSSL_API int  wc_RsaPublicKeyDecodeRaw(const byte* n, word32 nSz,
     \brief This function converts an RsaKey key to DER format.  The result is
     written to output and it returns the number of bytes written.
 
-    \return 0 Success
+    \return >0 Success, number of bytes written.
     \return BAD_FUNC_ARG Returned if key or output is null, or if key->type
     is not RSA_PRIVATE, or if inLen isn't large enough for output buffer.
     \return MEMORY_E Returned if there is an error allocating memory.
@@ -1069,7 +1162,7 @@ WOLFSSL_API int  wc_RsaPublicKeyDecodeRaw(const byte* n, word32 nSz,
     // Allocate memory for der
     int derSz = // Amount of memory allocated for der;
     RsaKey key;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     long e = 65537; // standard value to use for exponent
     ret = wc_MakeRsaKey(&key, 2048, e, &rng); // generate 2048 bit long
     private key
@@ -1086,7 +1179,7 @@ WOLFSSL_API int  wc_RsaPublicKeyDecodeRaw(const byte* n, word32 nSz,
     \sa wc_MakeRsaKey
     \sa wc_InitRng
 */
-WOLFSSL_API int wc_RsaKeyToDer(RsaKey*, byte* output, word32 inLen);
+int wc_RsaKeyToDer(RsaKey* key, byte* output, word32 inLen);
 
 /*!
     \ingroup RSA
@@ -1112,7 +1205,7 @@ WOLFSSL_API int wc_RsaKeyToDer(RsaKey*, byte* output, word32 inLen);
 
     _Example_
     \code
-    WC_WC_WC_RNG rng;
+    WC_RNG rng;
     RsaKey key;
     byte in[] = “I use Turing Machines to ask questions”
     byte out[256];
@@ -1129,9 +1222,9 @@ WOLFSSL_API int wc_RsaKeyToDer(RsaKey*, byte* output, word32 inLen);
     \sa wc_RsaPublicEncrypt
     \sa wc_RsaPrivateDecrypt_ex
 */
-WOLFSSL_API int  wc_RsaPublicEncrypt_ex(const byte* in, word32 inLen, byte* out,
+int  wc_RsaPublicEncrypt_ex(const byte* in, word32 inLen, byte* out,
                    word32 outLen, RsaKey* key, WC_RNG* rng, int type,
-                   enum wc_HashType hash, int mgf, byte* label, word32 lableSz);
+                   enum wc_HashType hash, int mgf, byte* label, word32 labelSz);
 
 /*!
     \ingroup RSA
@@ -1159,7 +1252,7 @@ WOLFSSL_API int  wc_RsaPublicEncrypt_ex(const byte* in, word32 inLen, byte* out,
 
     _Example_
     \code
-    WC_WC_WC_RNG rng;
+    WC_RNG rng;
     RsaKey key;
     byte in[] = “I use Turing Machines to ask questions”
     byte out[256];
@@ -1182,9 +1275,9 @@ WOLFSSL_API int  wc_RsaPublicEncrypt_ex(const byte* in, word32 inLen, byte* out,
 
     \sa none
 */
-WOLFSSL_API int  wc_RsaPrivateDecrypt_ex(const byte* in, word32 inLen,
+int  wc_RsaPrivateDecrypt_ex(const byte* in, word32 inLen,
                    byte* out, word32 outLen, RsaKey* key, int type,
-                   enum wc_HashType hash, int mgf, byte* label, word32 lableSz);
+                   enum wc_HashType hash, int mgf, byte* label, word32 labelSz);
 
 /*!
     \ingroup RSA
@@ -1216,7 +1309,7 @@ WOLFSSL_API int  wc_RsaPrivateDecrypt_ex(const byte* in, word32 inLen,
 
     _Example_
     \code
-    WC_WC_WC_RNG rng;
+    WC_RNG rng;
     RsaKey key;
     byte in[] = “I use Turing Machines to ask questions”
     byte out[256];
@@ -1240,9 +1333,9 @@ WOLFSSL_API int  wc_RsaPrivateDecrypt_ex(const byte* in, word32 inLen,
 
     \sa none
 */
-WOLFSSL_API int  wc_RsaPrivateDecryptInline_ex(byte* in, word32 inLen,
+int  wc_RsaPrivateDecryptInline_ex(byte* in, word32 inLen,
                       byte** out, RsaKey* key, int type, enum wc_HashType hash,
-                      int mgf, byte* label, word32 lableSz);
+                      int mgf, byte* label, word32 labelSz);
 
 /*!
     \ingroup RSA
@@ -1284,10 +1377,9 @@ WOLFSSL_API int  wc_RsaPrivateDecryptInline_ex(byte* in, word32 inLen,
     \sa wc_InitRsaKey
     \sa wc_InitRsaKey_ex
     \sa wc_MakeRsaKey
-    \sa XMEMSET
 */
-WOLFSSL_API int  wc_RsaFlattenPublicKey(RsaKey*, byte*, word32*, byte*,
-                                                                       word32*);
+int  wc_RsaFlattenPublicKey(RsaKey* key, byte* e, word32* eSz, byte* n,
+                            word32* nSz);
 
 /*!
     \ingroup RSA
@@ -1301,28 +1393,65 @@ WOLFSSL_API int  wc_RsaFlattenPublicKey(RsaKey*, byte*, word32*, byte*,
     \return <0 Error
 
     \param key The RSA key structure to convert.
-    \param output Output buffer to hold DER.
+    \param output Output buffer to hold DER. (if NULL will return length only)
     \param inLen Length of buffer.
 
     _Example_
     \code
     RsaKey key;
 
-    wc_RsaInitKey(&key, NULL);
+    wc_InitRsaKey(&key, NULL);
     // Use key
 
-    int BUFFER_SIZE = // Some adequate size for the buffer
+    const int BUFFER_SIZE = 1024; // Some adequate size for the buffer
     byte output[BUFFER_SIZE];
-    if(wc_RsaKeyToPublicDer(&key, output, sizeof(output)) != 0)
-    {
+    if (wc_RsaKeyToPublicDer(&key, output, sizeof(output)) != 0) {
         // Handle Error
     }
     \endcode
 
-    \sa wc_RsaKeyToPublicDer
-    \sa wc_RsaInitKey
+    \sa wc_RsaPublicKeyDerSize
+    \sa wc_RsaKeyToPublicDer_ex
+    \sa wc_InitRsaKey
 */
-WOLFSSL_API int wc_RsaKeyToPublicDer(RsaKey*, byte* output, word32 inLen);
+int wc_RsaKeyToPublicDer(RsaKey* key, byte* output, word32 inLen);
+
+/*!
+    \ingroup RSA
+
+    \brief Convert RSA Public key to DER format. Writes to output, and
+    returns count of bytes written. If with_header is 0 then only the
+    ( seq + n + e) is returned in ASN.1 DER format and will exclude the header.
+
+    \return >0 Success, number of bytes written.
+    \return BAD_FUNC_ARG Returned if key or output is null.
+    \return MEMORY_E Returned when an error allocating memory occurs.
+    \return <0 Error
+
+    \param key The RSA key structure to convert.
+    \param output Output buffer to hold DER. (if NULL will return length only)
+    \param inLen Length of buffer.
+
+    _Example_
+    \code
+    RsaKey key;
+
+    wc_InitRsaKey(&key, NULL);
+    // Use key
+
+    const int BUFFER_SIZE = 1024; // Some adequate size for the buffer
+    byte output[BUFFER_SIZE];
+    if (wc_RsaKeyToPublicDer_ex(&key, output, sizeof(output), 0) != 0) {
+        // Handle Error
+    }
+    \endcode
+
+    \sa wc_RsaPublicKeyDerSize
+    \sa wc_RsaKeyToPublicDer
+    \sa wc_InitRsaKey
+*/
+int wc_RsaKeyToPublicDer_ex(RsaKey* key, byte* output, word32 inLen,
+    int with_header);
 
 /*!
     \ingroup RSA
@@ -1368,7 +1497,7 @@ WOLFSSL_API int wc_RsaKeyToPublicDer(RsaKey*, byte* output, word32 inLen);
 
     \param key pointer to the RsaKey structure in which to store the
     generated private key
-    \param size desired keylenth, in bits. Required to be greater than
+    \param size desired key length, in bits. Required to be greater than
     RSA_MIN_SIZE and less than RSA_MAX_SIZE
     \param e exponent parameter to use for generating the key. A secure
     choice is 65537
@@ -1378,7 +1507,7 @@ WOLFSSL_API int wc_RsaKeyToPublicDer(RsaKey*, byte* output, word32 inLen);
     _Example_
     \code
     RsaKey priv;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     int ret = 0;
     long e = 65537; // standard value to use for exponent
 
@@ -1393,7 +1522,7 @@ WOLFSSL_API int wc_RsaKeyToPublicDer(RsaKey*, byte* output, word32 inLen);
 
     \sa none
 */
-WOLFSSL_API int wc_MakeRsaKey(RsaKey* key, int size, long e, WC_RNG* rng);
+int wc_MakeRsaKey(RsaKey* key, int size, long e, WC_RNG* rng);
 
 /*!
     \ingroup RSA
@@ -1415,7 +1544,7 @@ WOLFSSL_API int wc_MakeRsaKey(RsaKey* key, int size, long e, WC_RNG* rng);
     RsaKey key;
     RsaNb  nb;
 
-    wc_RsaInitKey(&key, NULL);
+    wc_InitRsaKey(&key, NULL);
 
     // Enable non-blocking RSA mode - provide context
     ret = wc_RsaSetNonBlock(key, &nb);
@@ -1438,7 +1567,7 @@ WOLFSSL_API int wc_MakeRsaKey(RsaKey* key, int size, long e, WC_RNG* rng);
 
     \sa wc_RsaSetNonBlockTime
 */
-WOLFSSL_API int wc_RsaSetNonBlock(RsaKey* key, RsaNb* nb);
+int wc_RsaSetNonBlock(RsaKey* key, RsaNb* nb);
 
 /*!
     \ingroup RSA
@@ -1462,7 +1591,7 @@ WOLFSSL_API int wc_RsaSetNonBlock(RsaKey* key, RsaNb* nb);
     RsaKey key;
     RsaNb  nb;
 
-    wc_RsaInitKey(&key, NULL);
+    wc_InitRsaKey(&key, NULL);
     wc_RsaSetNonBlock(key, &nb);
     wc_RsaSetNonBlockTime(&key, 4000, 160); // Block Max = 4 ms, CPU = 160MHz
 
@@ -1470,5 +1599,5 @@ WOLFSSL_API int wc_RsaSetNonBlock(RsaKey* key, RsaNb* nb);
 
     \sa wc_RsaSetNonBlock
 */
-WOLFSSL_API int wc_RsaSetNonBlockTime(RsaKey* key, word32 maxBlockUs,
+int wc_RsaSetNonBlockTime(RsaKey* key, word32 maxBlockUs,
     word32 cpuMHz);
