@@ -149,14 +149,14 @@ static int FSPSM_HashInit(wolfssl_FSPSM_Hash* hash, void* heap, int devId,
     XMEMSET(hash, 0, sizeof(wolfssl_FSPSM_Hash));
     hash->sha_type = sha_type;
     hash->heap = heap;
-    
+
 #if defined(WOLFSSL_RENESAS_SCEPROTECT)
     hash->len  = 0;
     hash->used = 0;
     hash->msg  = NULL;
-    
+
 #elif defined(WOLFSSL_RENESAS_RSIP)
-    
+
     switch(hash->sha_type) {
     case FSPSM_SHA1:
         Init = FSPSM_SHA1_Init;
@@ -244,7 +244,7 @@ static int FSPSM_HashUpdate(wolfssl_FSPSM_Hash* hash,
     XMEMCPY(hash->msg + hash->used, data , sz);
     hash->used += sz;
 #elif defined(WOLFSSL_RENESAS_RSIP)
-    
+
     switch(hash->sha_type) {
     case FSPSM_SHA1:
         Update = FSPSM_SHA1_Up;
@@ -271,7 +271,7 @@ static int FSPSM_HashUpdate(wolfssl_FSPSM_Hash* hash,
         return BAD_FUNC_ARG;
     }
     wc_fspsm_hw_lock();
-    ret = Update(&hash->handle, data, sz);
+    ret = Update(&hash->handle, (byte*)data, sz);
     wc_fspsm_hw_unlock();
     return ret;
 #endif
@@ -309,7 +309,7 @@ static int FSPSM_HashFinal(wolfssl_FSPSM_Hash* hash, byte* out, word32 outSz)
         Final = FSPSM_SHA256_Final;
     } else
         return BAD_FUNC_ARG;
-    
+
     wc_fspsm_hw_lock();
 
     if (Init(&handle) == FSP_SUCCESS) {
@@ -328,7 +328,7 @@ static int FSPSM_HashFinal(wolfssl_FSPSM_Hash* hash, byte* out, word32 outSz)
         }
     }
     wc_fspsm_hw_unlock();
-    
+
 #elif defined(WOLFSSL_RENESAS_RSIP)
     switch(hash->sha_type) {
     case FSPSM_SHA1:
@@ -355,7 +355,7 @@ static int FSPSM_HashFinal(wolfssl_FSPSM_Hash* hash, byte* out, word32 outSz)
     default:
         return BAD_FUNC_ARG;
     }
-    
+
     wc_fspsm_hw_lock();
     ret = Final(&hash->handle, out, (uint32_t*)&sz);
     if (ret != FSP_SUCCESS) {
@@ -380,7 +380,7 @@ static int FSPSM_HashGet(wolfssl_FSPSM_Hash* hash, byte* out, word32 outSz)
     fsp_err_t (*Final )(FSPSM_SHA_HANDLE*, uint8_t*, uint32_t*);
     uint32_t sz = 0;
     (void) outSz;
-    
+
 #if defined(WOLFSSL_RENESAS_SCEPROTECT)
     FSPSM_SHA_HANDLE handle;
     fsp_err_t (*Init)(FSPSM_SHA_HANDLE*);
@@ -401,7 +401,7 @@ static int FSPSM_HashGet(wolfssl_FSPSM_Hash* hash, byte* out, word32 outSz)
         Final = FSPSM_SHA256_Final;
     } else
         return BAD_FUNC_ARG;
-    
+
     wc_fspsm_hw_lock();
     if (Init(&handle) == FSP_SUCCESS) {
         ret = Update(&handle, (uint8_t*)hash->msg, hash->used);
@@ -419,7 +419,7 @@ static int FSPSM_HashGet(wolfssl_FSPSM_Hash* hash, byte* out, word32 outSz)
         }
     }
     wc_fspsm_hw_unlock();
-    
+
 #elif defined(WOLFSSL_RENESAS_RSIP)
     switch(hash->sha_type) {
     case FSPSM_SHA1:
@@ -446,8 +446,8 @@ static int FSPSM_HashGet(wolfssl_FSPSM_Hash* hash, byte* out, word32 outSz)
     default:
         return BAD_FUNC_ARG;
     }
-    
-    
+
+
     if(FSPSM_HashCopy(hash, &hashCopy) != 0) {
         WOLFSSL_MSG("ShaCopy operation failed");
         WOLFSSL_ERROR(WC_HW_E);
@@ -461,7 +461,7 @@ static int FSPSM_HashGet(wolfssl_FSPSM_Hash* hash, byte* out, word32 outSz)
         ret = WC_HW_E;
     }
     wc_fspsm_hw_unlock();
-    
+
 #endif
 
     return ret;
