@@ -539,7 +539,7 @@ int wolfSSL_STSAFE_CryptoDevCb(int devId, wc_CryptoInfo* info, void* ctx)
                 &otherKeyY[0], (word32*)&otherKeyY_len);
             if (rc == 0) {
                 /* Compute shared secret */
-            	*info->pk.ecdh.outlen = 0;
+                *info->pk.ecdh.outlen = 0;
                 rc = stsafe_interface_shared_secret(
         #ifdef WOLFSSL_STSAFE_TAKES_SLOT
                     STSAFE_A_SLOT_0,
@@ -559,7 +559,7 @@ int wolfSSL_STSAFE_CryptoDevCb(int devId, wc_CryptoInfo* info, void* ctx)
 #endif /* HAVE_ECC */
 
     /* need to return negative here for error */
-    if (rc != 0 && rc != CRYPTOCB_UNAVAILABLE) {
+    if (rc != 0 && rc != WC_NO_ERR_TRACE(CRYPTOCB_UNAVAILABLE)) {
         WOLFSSL_MSG("STSAFE: CryptoCb failed");
     #ifdef USE_STSAFE_VERBOSE
         STSAFE_INTERFACE_PRINTF("STSAFE: CryptoCb failed %d\n", rc);
